
#include "transformation.h"
#include "x.h"
#include "xpex.h"
#include "defs.h"

#define DEGRAD( _a )  ( (double)( _a ) * 3.1415926 / 180.0 )

gcd( m, n )
int m, n;
{
   while( m != n ) {
      if( m > n ) m -= n;
      else        n -= m;
   }
   return m;
}


PEXMatrix anim_matrix;


PEXTranslate_( ch, val, matrix )
int ch;
double val;
PEXMatrix matrix;
{
    PEXVector *v;

    v = (PEXVector *)malloc(sizeof(PEXVector));
    v->x = v->y = v->z = 0.;
    switch( ch ) {
        case PEXXAxis:
            v->x = (float)val;
            break;
        case PEXYAxis:
            v->y = (float)val;
            break;
        case PEXZAxis:
            v->z = (float)val;
            break;
    }
    PEXTranslate( v, matrix );
}


PEXScale_( ch, val, matrix )
int ch;
double val;
PEXMatrix matrix;
{
    PEXVector *v;

    v = (PEXVector *)malloc(sizeof(PEXVector));
    v->x = v->y = v->z = 1.;
    switch( ch ) {
        case PEXXAxis:
            v->x = (float)val;
            break;
        case PEXYAxis:
            v->y = (float)val;
            break;
        case PEXZAxis:
            v->z = (float)val;
            break;
    }
    PEXScale( v, matrix );
}


redisplay()
{
    extern void redraw();
    extern Display *display;
    extern PEXStructure object, object_shadow;

    PEXSetEditingMode( display, object, PEXStructureReplace );
    PEXSetElementPtr( display, object, PEXBeginning, 0 );
    PEXSetElementPtrAtLabel( display, object, ROTATION_LABEL, 1 );
    PEXSetLocalTransform( display, object, PEXOCStore,
         PEXReplace, anim_matrix );

    PEXSetEditingMode( display, object_shadow, PEXStructureReplace );
    PEXSetElementPtr( display, object_shadow, PEXBeginning, 0 );
    PEXSetElementPtrAtLabel( display, object_shadow, SHADOW_ROTATION_LABEL, 1 );
    PEXSetLocalTransform( display, object_shadow, PEXOCStore,
         PEXReplace, anim_matrix );

    redraw();
}


void execTrans( list )
AnimScript *list;
{
   AnimScript *anim;
   double eachStep1, eachStep2, eachStep3;
   register int i;
   PEXMatrix new_matrix;

   PEXIdentityMatrix( anim_matrix );

   anim = list;
   while( anim != NILanim ) {
      eachStep1 = anim->move1 / (double)anim->step1;
      eachStep2 = anim->move2 / (double)anim->step2;
      eachStep3 = anim->move3 / (double)anim->step3;

      switch( anim->code ) {
/*rot*/
         case ROT_X:
            for(i=1; i<=anim->step1; i++) {
               PEXRotate( PEXXAxis, DEGRAD(eachStep1), new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
         case ROT_Y:
            for(i=1; i<=anim->step1; i++) {
               PEXRotate( PEXYAxis, DEGRAD(eachStep1), new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
         case ROT_Z:
            for(i=1; i<=anim->step1; i++) {
               PEXRotate( PEXZAxis, DEGRAD(eachStep1), new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
/*trans*/
         case TRANS_X:
            for(i=1; i<=anim->step1; i++) {
               PEXTranslate_( PEXXAxis, eachStep1, new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
         case TRANS_Y:
            for(i=1; i<=anim->step1; i++) {
               PEXTranslate_( PEXYAxis, eachStep1, new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
         case TRANS_Z:
            for(i=1; i<=anim->step1; i++) {
               PEXTranslate_( PEXZAxis, eachStep1, new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
/*scale*/
         case SCALE_X:
            for(i=1; i<=anim->step1; i++) {
               PEXScale_( PEXXAxis, 1.-eachStep1, new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
         case SCALE_Y:
            for(i=1; i<=anim->step1; i++) {
               PEXScale_( PEXYAxis, 1.-eachStep1, new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;
         case SCALE_Z:
            for(i=1; i<=anim->step1; i++) {
               PEXScale_( PEXZAxis, 1.-eachStep1, new_matrix );
               PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
               redisplay();
            }
            break;


/*rot*/
         case ROT_XY:
            two_axis_rot( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXXAxis, PEXYAxis );
            break;
         case ROT_YZ:
            two_axis_rot( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXYAxis, PEXZAxis );
            break;
         case ROT_ZX:
            two_axis_rot( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXZAxis, PEXXAxis );
            break;
/*trans*/
         case TRANS_XY:
            two_axis_trn( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXXAxis ,PEXYAxis );
            break;
         case TRANS_YZ:
            two_axis_trn( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXYAxis ,PEXZAxis );
            break;
         case TRANS_ZX:
            two_axis_trn( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXZAxis, PEXXAxis );
            break;
/*scale*/
         case SCALE_XY:
            two_axis_scl( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXXAxis, PEXYAxis );
            break;
         case SCALE_YZ:
            two_axis_scl( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXYAxis, PEXZAxis );
            break;
         case SCALE_ZX:
            two_axis_scl( anim->step1, anim->step2, eachStep1, eachStep2, 
                          PEXZAxis, PEXXAxis );
            break;


/*rot*/
         case ROT_XYZ:
            three_axis_rot( anim->step1, anim->step2, anim->step3,
                            eachStep1, eachStep2, eachStep3,
                            PEXXAxis, PEXYAxis, PEXZAxis );
            break;
/*trans*/
         case TRANS_XYZ:
            three_axis_trn( anim->step1, anim->step2, anim->step3,
                            eachStep1, eachStep2, eachStep3,
                            PEXXAxis, PEXYAxis, PEXZAxis );
            break;
/*scale*/
         case SCALE_XYZ:
            three_axis_scl( anim->step1, anim->step2, anim->step3,
                            eachStep1, eachStep2, eachStep3,
                            PEXXAxis, PEXYAxis, PEXZAxis );
            break;
      }
      anim = anim->next;
   }
}


two_axis_rot( step1, step2, eachStep1, eachStep2, ch1, ch2 )
int step1, step2;
double eachStep1, eachStep2;
int ch1, ch2;
{
   register int i;
   int GCD;
   int halfStep1, halfStep2;
   PEXMatrix m1, m2;
   PEXMatrix new_matrix;

   GCD = gcd( step1, step2 );
   halfStep1 = step1 / GCD;
   halfStep2 = step2 / GCD;

   for(i=1; i<=GCD; i++) {
      PEXRotate( ch1, DEGRAD(halfStep1*eachStep1), m1 );
      PEXRotate( ch2, DEGRAD(halfStep2*eachStep2), m2 );
      PEXMatrixMult( m1, m2, new_matrix );
      PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
      redisplay();
   }
}


two_axis_trn( step1, step2, eachStep1, eachStep2, ch1, ch2 )
int step1, step2;
double eachStep1, eachStep2;
int ch1, ch2;
{
   register int i;
   int GCD;
   int halfStep1, halfStep2;
   PEXMatrix m1, m2;
   PEXMatrix new_matrix;

   GCD = gcd( step1, step2 );
   halfStep1 = step1 / GCD;
   halfStep2 = step2 / GCD;

   for(i=1; i<=GCD; i++) {
      PEXTranslate_( ch1, (double)(halfStep1*eachStep1), m1 );
      PEXTranslate_( ch2, (double)(halfStep2*eachStep2), m2 );
      PEXMatrixMult( m1, m2, new_matrix );
      PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
      redisplay();
   }
}


two_axis_scl( step1, step2, eachStep1, eachStep2, ch1, ch2 )
int step1, step2;
double eachStep1, eachStep2;
int ch1, ch2;
{
   register int i;
   int GCD;
   int halfStep1, halfStep2;
   PEXMatrix m1, m2;
   PEXMatrix new_matrix;

   GCD = gcd( step1, step2 );
   halfStep1 = step1 / GCD;
   halfStep2 = step2 / GCD;

   for(i=1; i<=GCD; i++) {
      PEXScale_( ch1, 1.-(double)(halfStep1*eachStep1), m1 );
      PEXScale_( ch2, 1.-(double)(halfStep2*eachStep2), m2 );
      PEXMatrixMult( m1, m2, new_matrix );
      PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
      redisplay();
   }
}


three_axis_rot( step1,step2,step3, eachStep1,eachStep2,eachStep3, ch1,ch2,ch3 )
int step1, step2, step3;
double eachStep1, eachStep2, eachStep3;
int ch1, ch2, ch3;
{
   register int i;
   int GCD;
   int halfStep1, halfStep2, halfStep3;
   PEXMatrix m1, m2, m3;
   PEXMatrix new_matrix;

   GCD = gcd( step1, step2 );
   GCD = gcd( GCD, step3 );
   halfStep1 = step1 / GCD;
   halfStep2 = step2 / GCD;
   halfStep3 = step3 / GCD;

   for(i=1; i<=GCD; i++) {
      PEXRotate( ch1, DEGRAD(halfStep1*eachStep1), m1 );
      PEXRotate( ch2, DEGRAD(halfStep2*eachStep2), m2 );
      PEXRotate( ch3, DEGRAD(halfStep3*eachStep3), m3 );
      PEXMatrixMult( m2, m3, m2 );
      PEXMatrixMult( m1, m2, new_matrix );
      PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
      redisplay();
   }
}


three_axis_trn( step1,step2,step3, eachStep1,eachStep2,eachStep3, ch1,ch2,ch3 )
int step1, step2, step3;
double eachStep1, eachStep2, eachStep3;
int ch1, ch2, ch3;
{
   register int i;
   int GCD;
   int halfStep1, halfStep2, halfStep3;
   PEXMatrix m1, m2, m3;
   PEXMatrix new_matrix;

   GCD = gcd( step1, step2 );
   GCD = gcd( GCD, step3 );
   halfStep1 = step1 / GCD;
   halfStep2 = step2 / GCD;
   halfStep3 = step3 / GCD;

   for(i=1; i<=GCD; i++) {
      PEXTranslate_( ch1, (double)(halfStep1*eachStep1), m1 );
      PEXTranslate_( ch2, (double)(halfStep2*eachStep2), m2 );
      PEXTranslate_( ch3, (double)(halfStep3*eachStep3), m3 );
      PEXMatrixMult( m2, m3, m2 );
      PEXMatrixMult( m1, m2, new_matrix );
      PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
      redisplay();
   }
}


three_axis_scl( step1,step2,step3, eachStep1,eachStep2,eachStep3, ch1,ch2,ch3 )
int step1, step2, step3;
double eachStep1, eachStep2, eachStep3;
int ch1, ch2, ch3;
{
   register int i;
   int GCD;
   int halfStep1, halfStep2, halfStep3;
   PEXMatrix m1, m2, m3;
   PEXMatrix new_matrix;

   GCD = gcd( step1, step2 );
   GCD = gcd( GCD, step3 );
   halfStep1 = step1 / GCD;
   halfStep2 = step2 / GCD;
   halfStep3 = step3 / GCD;

   for(i=1; i<=GCD; i++) {
      PEXScale_( ch1, 1.-(double)(halfStep1*eachStep1), m1 );
      PEXScale_( ch2, 1.-(double)(halfStep2*eachStep2), m2 );
      PEXScale_( ch3, 1.-(double)(halfStep3*eachStep3), m3 );
      PEXMatrixMult( m2, m3, m2 );
      PEXMatrixMult( m1, m2, new_matrix );
      PEXMatrixMult( anim_matrix, new_matrix, anim_matrix );
      redisplay();
   }
}
