
#include "unix.h"
#include "x.h"
#include "xpex.h"
#include "off.h"
#include "defs.h"

Boolean WriteAOFF( filename )
char *filename;
{
    extern OFFObjDesc *Obj;
    extern PEXReflectionAttributes refl_props;
    extern PEXColor colorMaterial;
    extern int light_switches[4];
    extern PEXLightEntry lights[4];
    extern int numLightsON, numLightsOFF;
    extern PEXTableIndex lightsON[4], lightsOFF[4];

    OFFProperty *props;
    FILE *fp;
    char *filedir;

    filedir = (char *)malloc( strlen( getenv( "OBJ_PATH" ) ) +
        strlen( filename ) + 2 );
    strcpy( filedir, getenv( "OBJ_PATH" ) );
    strcat( filedir, "/" );
    strcat( filedir, filename );

    if( (fp = fopen( filedir, "w" )) == NULL ) {
        fclose( fp );
        return 0;
    }
    else {
        int polygon_colors_defined = 0, vertex_colors_defined = 0;
        float vertex_colors_red, vertex_colors_green, vertex_colors_blue;
        float polygon_colors_red, polygon_colors_green, polygon_colors_blue;

        props = Obj->FirstProp;
        while( props != NULL ) {
            if( (props->PropType == OFF_DEFAULT_DATA) &&
                !strcmp( props->PropName, "vertex_colors" ) ) {
                vertex_colors_defined = 1;
                vertex_colors_red   = * (float*)((char*)props->PropData);
                vertex_colors_green = *((float*)((char*)props->PropData)+1);
                vertex_colors_blue  = *((float*)((char*)props->PropData)+2);
            }
            else if( (props->PropType == OFF_DEFAULT_DATA) &&
                !strcmp( props->PropName, "polygon_colors" ) ) {
                polygon_colors_defined = 1;
                polygon_colors_red   = * (float*)((char*)props->PropData);
                polygon_colors_green = *((float*)((char*)props->PropData)+1);
                polygon_colors_blue  = *((float*)((char*)props->PropData)+2);
            }
            else if(
                strcmp( props->PropName, "material_colors" ) &&
                strcmp( props->PropName, "ambient_coef" ) &&
                strcmp( props->PropName, "diffuse_coef" ) &&
                strcmp( props->PropName, "specular_coef" ) &&
                strcmp( props->PropName, "specular_power" ) &&
                strcmp( props->PropName, "specular_color" ) &&
                strcmp( props->PropName, "transmission" ) &&
                strcmp( props->PropName, "ambient_on" ) &&
                strcmp( props->PropName, "ambient_color" ) &&
                strcmp( props->PropName, "direct_on" ) &&
                strcmp( props->PropName, "direct_direction" ) &&
                strcmp( props->PropName, "direct_color" ) &&
                strcmp( props->PropName, "position_on" ) &&
                strcmp( props->PropName, "position_point" ) &&
                strcmp( props->PropName, "position_atten1" ) &&
                strcmp( props->PropName, "position_atten2" ) &&
                strcmp( props->PropName, "position_color" ) &&
                strcmp( props->PropName, "spot_on" ) &&
                strcmp( props->PropName, "spot_point" ) &&
                strcmp( props->PropName, "spot_direction" ) &&
                strcmp( props->PropName, "spot_atten1" ) &&
                strcmp( props->PropName, "spot_atten2" ) &&
                strcmp( props->PropName, "spot_conc" ) &&
                strcmp( props->PropName, "spot_angle" ) &&
                strcmp( props->PropName, "spot_color" ) ) {

                fprintf( fp, "%s\t", props->PropName );

                if( props->PropType == OFF_DEFAULT_DATA ) {
                    if( strcmp( props->DataFormat, "s" ) == 0 )
                        fprintf( fp, "default\t\t%s\t\t%s\n",
                            props->DataFormat, *((char**)props->PropData) );
                    else if( strcmp( props->DataFormat, "f" ) == 0 )
                        fprintf( fp, "default\t\t%s\t\t%.3f\n",
                            props->DataFormat,
                            *(float*)((char*)props->PropData) );
                    else if( strcmp( props->DataFormat, "fff" ) == 0 )
                        fprintf( fp, "default\t\t%s\t\t%.3f %.3f %.3f\n",
                            props->DataFormat,
                            * (float*)((char*)props->PropData),
                            *((float*)((char*)props->PropData)+1),
                            *((float*)((char*)props->PropData)+2) );
                }
                else if( props->PropType == OFF_GENERIC_DATA )
                    fprintf( fp, "generic\t\t%s\t\t%s\n",
                        props->DataFormat, props->PropFileName );
                else if( props->PropType == OFF_INDEXED_DATA )
                    fprintf( fp, "indexed\t\t%s\t\t%s\n",
                        props->DataFormat, props->PropFileName );
                else if( props->PropType == OFF_INDEXED_POLY_DATA )
                    fprintf( fp, "indexed_poly\t%s\t\t%s\n",
                        props->DataFormat, props->PropFileName );
            }

            props = props->NextProp;
        }
        if( vertex_colors_defined )
        fprintf( fp, "vertex_colors\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            vertex_colors_red, vertex_colors_green, vertex_colors_blue );
        if( polygon_colors_defined )
        fprintf( fp, "polygon_colors\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            polygon_colors_red, polygon_colors_green, polygon_colors_blue );
        fprintf( fp, "material_colors\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            colorMaterial.rgb.red,
            colorMaterial.rgb.green,
            colorMaterial.rgb.blue );

        fprintf( fp, "ambient_coef\tdefault\t\tf\t\t%.3f\n",
            refl_props.ambient );
        fprintf( fp, "diffuse_coef\tdefault\t\tf\t\t%.3f\n",
            refl_props.diffuse );
        fprintf( fp, "specular_coef\tdefault\t\tf\t\t%.3f\n",
            refl_props.specular );
        fprintf( fp, "specular_power\tdefault\t\tf\t\t%.3f\n",
            refl_props.specular_conc );
        fprintf( fp, "specular_color\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            refl_props.specular_color.value.rgb.red,
            refl_props.specular_color.value.rgb.green,
            refl_props.specular_color.value.rgb.blue );
        fprintf( fp, "transmission\tdefault\t\tf\t\t%.3f\n",
            refl_props.transmission );

        fprintf( fp, "ambient_on\tdefault\t\tf\t\t%.3f\n",
            (light_switches[AMBIENT] ? 1.0 : 0.0 ) );
        fprintf( fp, "ambient_color\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[AMBIENT].color.value.rgb.red,
            lights[AMBIENT].color.value.rgb.green,
            lights[AMBIENT].color.value.rgb.blue );
        fprintf( fp, "direct_on\tdefault\t\tf\t\t%.3f\n",
            (light_switches[DIRECTIONAL] ? 1.0 : 0.0 ) );
        fprintf( fp, "direct_direction\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[DIRECTIONAL].direction.x,
            lights[DIRECTIONAL].direction.y,
            lights[DIRECTIONAL].direction.z );
        fprintf( fp, "direct_color\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[DIRECTIONAL].color.value.rgb.red,
            lights[DIRECTIONAL].color.value.rgb.green,
            lights[DIRECTIONAL].color.value.rgb.blue );
        fprintf( fp, "position_on\tdefault\t\tf\t\t%.3f\n",
            (light_switches[POSITIONAL] ? 1.0 : 0.0 ) );
        fprintf( fp, "position_point\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[POSITIONAL].point.x,
            lights[POSITIONAL].point.y,
            lights[POSITIONAL].point.z );
        fprintf( fp, "position_atten1\tdefault\t\tf\t\t%.3f\n",
            lights[POSITIONAL].attenuation1 );
        fprintf( fp, "position_atten2\tdefault\t\tf\t\t%.3f\n",
            lights[POSITIONAL].attenuation2 );
        fprintf( fp, "position_color\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[POSITIONAL].color.value.rgb.red,
            lights[POSITIONAL].color.value.rgb.green,
            lights[POSITIONAL].color.value.rgb.blue );
        fprintf( fp, "spot_on\tdefault\t\tf\t\t%.3f\n",
            (light_switches[SPOT] ? 1.0 : 0.0 ) );
        fprintf( fp, "spot_point\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[SPOT].point.x,
            lights[SPOT].point.y,
            lights[SPOT].point.z );
        fprintf( fp, "spot_direction\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[SPOT].direction.x,
            lights[SPOT].direction.y,
            lights[SPOT].direction.z );
        fprintf( fp, "spot_atten1\tdefault\t\tf\t\t%.3f\n",
            lights[SPOT].attenuation1 );
        fprintf( fp, "spot_atten2\tdefault\t\tf\t\t%.3f\n",
            lights[SPOT].attenuation2 );
        fprintf( fp, "spot_conc\tdefault\t\tf\t\t%.3f\n",
            lights[SPOT].concentration );
        fprintf( fp, "spot_angle\tdefault\t\tf\t\t%.3f\n",
            lights[SPOT].spread_angle );
        fprintf( fp, "spot_color\tdefault\t\tfff\t\t%.3f %.3f %.3f\n",
            lights[SPOT].color.value.rgb.red,
            lights[SPOT].color.value.rgb.green,
            lights[SPOT].color.value.rgb.blue );
        fclose( fp );
        return 1;
    }
}
